@@ -60,7 +60,7 @@
 #include "solaris_pixdata.h"
 #include "thermal_pixdata.h"
 #include "tv_pixdata.h"
-#include "vcsc_pixdata.h"
+#include "vcs_pixdata.h"
 #include "x_pixdata.h"
 #include "xvideo_pixdata.h"
 
@@ -375,7 +375,7 @@ static gboolean select_artwork(BannerArtworkType artwork,
         { BANNER_ARTWORK_SOLARIS,        TRUE,  16, &solaris_pixdata        },
         { BANNER_ARTWORK_THERMAL,        FALSE, 16, &thermal_pixdata        },
         { BANNER_ARTWORK_TV,             FALSE, 16, &tv_pixdata             },
-        { BANNER_ARTWORK_VCSC,           FALSE, 16, &vcsc_pixdata           },
+        { BANNER_ARTWORK_VCS,            FALSE, 16, &vcs_pixdata            },
         { BANNER_ARTWORK_X,              FALSE, 16, &x_pixdata              },
         { BANNER_ARTWORK_XVIDEO,         FALSE, 16, &xvideo_pixdata         },
         { BANNER_ARTWORK_INVALID,        FALSE, 16, NULL                    },
