@@ -124,12 +124,6 @@ G_BEGIN_DECLS
 (!strcmp(( m )->data.identifier, "nvidia-auto-select"))
 
 
-/* Calculates the vertical refresh rate of the modeline in Hz */
-#define GET_MODELINE_REFRESH_RATE(m)                         \
-(((double)((m)->data.clock) * 1000) /                        \
- ((double)((m)->data.htotal) * (double)((m)->data.vtotal)))
-
-
 /* Calculates the horizontal refresh rate (sync) of the modeline in kHz */
 #define GET_MODELINE_HSYNC(m)                                        \
 (((double)((m)->data.clock)) / (2.0f * (double)((m)->data.htotal)))
@@ -145,10 +139,12 @@ typedef struct nvModeLineRec {
 
     XConfigModeLineRec data; /* Modeline information */
 
+    double refresh_rate; /* in Hz */
+
     /* Extra information */
     unsigned int source;
     char *xconfig_name;
-    
+
 } nvModeLine, *nvModeLinePtr;
 
 
@@ -279,6 +275,7 @@ typedef struct nvGpuRec {
     int max_width;
     int max_height;
     int max_displays;
+    Bool allow_depth_30;
 
     char *name;  /* Name of the GPU */
 
