@@ -135,10 +135,19 @@ static const char * __hsync_delay_help =
 "The HSync Delay entry allows you to specify the horizontal delay between the "
 "input signal and the output signal generated by the SDI device.";
 
+static const char * __hsync_advance_help =
+"The HSync Advance entry allows you to specify the horizontal advance between "
+"the input signal and the output signal generated by the SDI device.";
+
 static const char * __vsync_delay_help =
 "The VSync Delay entry allows you to specify the vertical delay between the "
 "input signal and the output signal generated by the SDI device.";
 
+static const char * __vsync_advance_help =
+"The VSync Advance entry allows you to specify the vertical advance between "
+"the input signal and the output signal generated by the SDI device.";
+
+
 
 
 /**** Utility Functions ******************************************************/
@@ -329,6 +338,8 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
     gint val, i, width, height;
     ReturnStatus ret;
     gint row;
+
+    const char *help_text;
     
 
     /* make sure we have a handle */
@@ -540,7 +551,7 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
 
 
     /*
-     * Synchronization Delay
+     * Synchronization Delay/Advance
      */
     
     ctk_gvo_get_video_format_resolution(ctk_gvo_sync->input_video_format,
@@ -548,7 +559,12 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
 
     /* NV_CTRL_GVO_SYNC_DELAY_PIXELS */
 
-    label = gtk_label_new("HSync Delay:");
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        label = gtk_label_new("HSync Advance:");
+    } else {
+        label = gtk_label_new("HSync Delay:");
+    }
+
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row+1,
                      GTK_FILL, GTK_FILL, TABLE_PADDING, TABLE_PADDING);
@@ -558,11 +574,19 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
 
     if (width < 1) width = 1;
 
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        width = INT_MAX;
+        help_text = __hsync_advance_help;
+    } else {
+        help_text = __hsync_delay_help;
+    }
+
     ctk_gvo_sync->hsync_delay_spin_button =
         gtk_spin_button_new_with_range(0.0, width, 1);
 
-    ctk_config_set_tooltip(ctk_config, ctk_gvo_sync->hsync_delay_spin_button,
-                           __hsync_delay_help);
+    ctk_config_set_tooltip(ctk_config,
+                           ctk_gvo_sync->hsync_delay_spin_button,
+                           help_text);
 
     gtk_spin_button_set_value
         (GTK_SPIN_BUTTON(ctk_gvo_sync->hsync_delay_spin_button), val);
@@ -588,7 +612,11 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
 
     /* NV_CTRL_GVO_SYNC_DELAY_LINES */
     
-    label = gtk_label_new("VSync Delay:");
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        label = gtk_label_new("VSync Advance:");
+    } else {
+        label = gtk_label_new("VSync Delay:");
+    }
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row+1,
                      GTK_FILL, GTK_FILL, TABLE_PADDING, TABLE_PADDING);
@@ -598,12 +626,19 @@ GtkWidget* ctk_gvo_sync_new(NvCtrlAttributeHandle *handle,
     
     if (height < 1) height = 1;
 
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        height = INT_MAX;
+        help_text = __vsync_advance_help;
+    } else {
+        help_text = __vsync_delay_help;
+    }
+
     ctk_gvo_sync->vsync_delay_spin_button =
         gtk_spin_button_new_with_range(0.0, height, 1);
-    
-    ctk_config_set_tooltip(ctk_config, ctk_gvo_sync->vsync_delay_spin_button,
-                           __vsync_delay_help);
-
+   
+    ctk_config_set_tooltip(ctk_config,
+                           ctk_gvo_sync->vsync_delay_spin_button,
+                           help_text);
     gtk_spin_button_set_value
         (GTK_SPIN_BUTTON(ctk_gvo_sync->vsync_delay_spin_button), val);
     
@@ -1014,6 +1049,11 @@ static void update_delay_spin_buttons_range(CtkGvoSync *ctk_gvo_sync)
 {
     gint w, h;
     
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        // No need to reset max values
+        return;
+    }
+
     ctk_gvo_get_video_format_resolution(ctk_gvo_sync->input_video_format,
                                         &w, &h);
 
@@ -1021,7 +1061,7 @@ static void update_delay_spin_buttons_range(CtkGvoSync *ctk_gvo_sync)
         (GTK_SPIN_BUTTON(ctk_gvo_sync->hsync_delay_spin_button), 0, w);
     gtk_spin_button_set_range
         (GTK_SPIN_BUTTON(ctk_gvo_sync->vsync_delay_spin_button), 0, h);
-    
+
 } /* update_delay_spin_buttons_range() */
 
 
@@ -1498,10 +1538,18 @@ GtkTextBuffer* ctk_gvo_sync_create_help(GtkTextTagTable *table,
     ctk_help_para(b, &i, __sync_format_help);
     ctk_help_heading(b, &i, "Sync Status");
     ctk_help_para(b, &i, __sync_status_help);
-    ctk_help_heading(b, &i, "HSync Delay");
-    ctk_help_para(b, &i, __hsync_delay_help);
-    ctk_help_heading(b, &i, "VSync Delay");
-    ctk_help_para(b, &i, __vsync_delay_help);
+
+    if ( ctk_gvo_sync->caps && NV_CTRL_GVO_CAPABILITIES_ADVANCE_SYNC_SKEW ) {
+        ctk_help_heading(b, &i, "HSync Advance");
+        ctk_help_para(b, &i, __hsync_advance_help);
+        ctk_help_heading(b, &i, "VSync Advance");
+        ctk_help_para(b, &i, __vsync_advance_help);
+    } else {
+        ctk_help_heading(b, &i, "HSync Delay");
+        ctk_help_para(b, &i, __hsync_delay_help);
+        ctk_help_heading(b, &i, "VSync Delay");
+        ctk_help_para(b, &i, __vsync_delay_help);
+    }
 
     ctk_help_finish(b);
 
