@@ -34,14 +34,24 @@
 
 static void vblank_sync_button_toggled   (GtkWidget *, gpointer);
 
+static void post_vblank_sync_button_toggled(CtkOpenGL *, gboolean);
+
+static void post_allow_flipping_button_toggled(CtkOpenGL *, gboolean);
+
+static void post_force_stereo_button_toggled(CtkOpenGL *, gboolean);
+
+static void post_show_sli_hud_button_toggled(CtkOpenGL *, gboolean);
+
+static void post_xinerama_stereo_button_toggled(CtkOpenGL *, gboolean);
+
+static void post_aa_line_gamma_toggled(CtkOpenGL *, gboolean);
+
 static void allow_flipping_button_toggled(GtkWidget *, gpointer);
 
 static void force_stereo_button_toggled (GtkWidget *, gpointer);
 
 static void xinerama_stereo_button_toggled (GtkWidget *, gpointer);
 
-static void force_generic_cpu_toggled    (GtkWidget *, gpointer);
-
 static void aa_line_gamma_toggled        (GtkWidget *, gpointer);
 
 static void show_sli_hud_button_toggled (GtkWidget *, gpointer);
@@ -68,16 +78,6 @@ static const char *__sync_to_vblank_help =
 "applied to OpenGL applications that are started after "
 "this option is set.";
 
-static const char *__force_generic_cpu_help =
-"Enable this option to disable use of CPU "
-"specific features such as MMX, SSE, or 3DNOW!.  "
-"Use of this option may result in performance "
-"loss, but may be useful in conjunction with "
-"software such as the Valgrind memory "
-"debugger.  This option is applied to "
-"OpenGL applications that are started after "
-"this option is set.";
-
 static const char *__image_settings_slider_help =
 "The Image Settings slider controls the image quality setting.";
 
@@ -158,7 +158,6 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     NVCTRLAttributeValidValuesRec image_settings_valid;
     gint image_settings_value;
     gint aa_line_gamma;
-    gint force_generic_cpu;
     gint show_sli_hud;
 
     ReturnStatus ret_sync_to_vblank;
@@ -167,7 +166,6 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     ReturnStatus ret_xinerama_stereo;
     ReturnStatus ret_image_settings;
     ReturnStatus ret_aa_line_gama;
-    ReturnStatus ret_force_generic_cpu;
     ReturnStatus ret_show_sli_hud;
 
     /* Query OpenGL settings */
@@ -199,11 +197,6 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
 
     ret_aa_line_gama = NvCtrlGetAttribute(handle, NV_CTRL_OPENGL_AA_LINE_GAMMA,
                                           &aa_line_gamma);
-
-    ret_force_generic_cpu = NvCtrlGetAttribute(handle,
-                                               NV_CTRL_FORCE_GENERIC_CPU,
-                                               &force_generic_cpu);
-
     ret_show_sli_hud = NvCtrlGetAttribute(handle, NV_CTRL_SHOW_SLI_HUD,
                                           &show_sli_hud);
 
@@ -214,7 +207,6 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         (ret_xinerama_stereo != NvCtrlSuccess) &&
         (ret_image_settings != NvCtrlSuccess) &&
         (ret_aa_line_gama != NvCtrlSuccess) &&
-        (ret_force_generic_cpu != NvCtrlSuccess) &&
         (ret_show_sli_hud != NvCtrlSuccess)) {
         return NULL;
     }
@@ -279,10 +271,10 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         g_signal_connect(G_OBJECT(ctk_event),
                          CTK_EVENT_NAME(NV_CTRL_SYNC_TO_VBLANK),
                          G_CALLBACK(value_changed), (gpointer) ctk_opengl);
- 
+
         ctk_config_set_tooltip(ctk_config, check_button,
                                __sync_to_vblank_help);
-    
+
         ctk_opengl->active_attributes |= __SYNC_TO_VBLANK;
 
         ctk_opengl->sync_to_vblank_button = check_button;
@@ -366,7 +358,7 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
                          (gpointer) ctk_opengl);
 
         g_signal_connect(G_OBJECT(ctk_event),
-                         CTK_EVENT_NAME(NV_CTRL_FORCE_STEREO),
+                         CTK_EVENT_NAME(NV_CTRL_XINERAMA_STEREO),
                          G_CALLBACK(value_changed), (gpointer) ctk_opengl);
 
         ctk_config_set_tooltip(ctk_config, check_button, __xinerama_stereo_help);
@@ -469,38 +461,6 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         ctk_opengl->aa_line_gamma_button = check_button;
     }
     
-    /*
-     * Force Generic CPU
-     */
-
-    if (ret_force_generic_cpu == NvCtrlSuccess) {
-        label = gtk_label_new("Disable use of enhanced CPU instruction sets");
-
-        check_button = gtk_check_button_new();
-        gtk_container_add(GTK_CONTAINER(check_button), label);
-
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
-                                     force_generic_cpu ==
-                                     NV_CTRL_FORCE_GENERIC_CPU_ENABLE);
-
-        gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
-
-        g_signal_connect(G_OBJECT(check_button), "toggled",
-                         G_CALLBACK(force_generic_cpu_toggled),
-                         (gpointer) ctk_opengl);
-
-        g_signal_connect(G_OBJECT(ctk_event),
-                         CTK_EVENT_NAME(NV_CTRL_FORCE_GENERIC_CPU),
-                         G_CALLBACK(value_changed), (gpointer) ctk_opengl);
-        
-        ctk_config_set_tooltip(ctk_config, check_button,
-                               __force_generic_cpu_help);
-
-        ctk_opengl->active_attributes |= __FORCE_GENERIC_CPU;
-
-        ctk_opengl->force_generic_cpu_button = check_button;
-    }
-
     if (ret_show_sli_hud == NvCtrlSuccess) {
 
         label = gtk_label_new("Enable SLI Heads-Up-Display");
@@ -534,6 +494,59 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     return GTK_WIDGET(object);
 }
 
+/* 
+ * Prints status bar message
+ */
+static void post_vblank_sync_button_toggled(CtkOpenGL *ctk_opengl, 
+                                            gboolean enabled)
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL Sync to VBlank %s.",
+                                 enabled ? "enabled" : "disabled");
+}
+
+static void post_allow_flipping_button_toggled(CtkOpenGL *ctk_opengl, 
+                                                gboolean enabled) 
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL Flipping %s.",
+                                 enabled ? "allowed" : "not allowed");
+}
+
+static void post_force_stereo_button_toggled(CtkOpenGL *ctk_opengl, 
+                                             gboolean enabled)
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL Stereo Flipping %s.",
+                                 enabled ? "forced" : "not forced");
+}
+
+static void post_show_sli_hud_button_toggled(CtkOpenGL *ctk_opengl, 
+                                             gboolean enabled) 
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL SLI HUD %s.",
+                                 enabled ? "enabled" : "disabled");
+}
+
+static void post_xinerama_stereo_button_toggled(CtkOpenGL *ctk_opengl, 
+                                                gboolean enabled) 
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL Xinerama Stereo Flipping %s.",
+                                 enabled ? "allowed" : "not allowed");
+}
+
+static void post_aa_line_gamma_toggled(CtkOpenGL *ctk_opengl, 
+                                       gboolean enabled) 
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "OpenGL gamma correction for antialiased "
+                                 "lines %s.",
+                                 enabled ? "enabled" : "disabled");
+}
+
+
 static void vblank_sync_button_toggled(
     GtkWidget *widget,
     gpointer user_data
@@ -548,9 +561,7 @@ static void vblank_sync_button_toggled(
 
     NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_SYNC_TO_VBLANK, enabled);
 
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL Sync to VBlank %s.",
-                                 enabled ? "enabled" : "disabled");
+    post_vblank_sync_button_toggled(ctk_opengl, enabled);
 }
 
 
@@ -563,12 +574,9 @@ static void allow_flipping_button_toggled(GtkWidget *widget,
     ctk_opengl = CTK_OPENGL(user_data);
 
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-
-    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_FLIPPING_ALLOWED, enabled);
     
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL Flipping %s.",
-                                 enabled ? "allowed" : "prohibited");
+    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_FLIPPING_ALLOWED, enabled);
+    post_allow_flipping_button_toggled(ctk_opengl, enabled);
     
 }
 
@@ -583,10 +591,7 @@ static void force_stereo_button_toggled(GtkWidget *widget,
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_FORCE_STEREO, enabled);
-    
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL Stereo Flipping %s.",
-                                 enabled ? "forced" : "not forced");
+    post_force_stereo_button_toggled(ctk_opengl, enabled);
 }
 
 static void show_sli_hud_button_toggled(GtkWidget *widget,
@@ -600,10 +605,7 @@ static void show_sli_hud_button_toggled(GtkWidget *widget,
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_SHOW_SLI_HUD, enabled);
-
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL SLI HUD %s.",
-                                 enabled ? "enabled" : "disabled");
+    post_show_sli_hud_button_toggled(ctk_opengl, enabled); 
 }
 
 static void xinerama_stereo_button_toggled(GtkWidget *widget,
@@ -617,39 +619,7 @@ static void xinerama_stereo_button_toggled(GtkWidget *widget,
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_XINERAMA_STEREO, enabled);
-    
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL Xinerama Stereo Flipping %s.",
-                                 enabled ? "allowed" : "not allowed");
-}
-
-static void force_generic_cpu_toggled(
-    GtkWidget *widget,
-    gpointer user_data
-)
-{
-    CtkOpenGL *ctk_opengl;
-    gboolean enabled;
-    ReturnStatus ret;
-    
-    ctk_opengl = CTK_OPENGL(user_data);
-
-    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-
-    ret = NvCtrlSetAttribute(ctk_opengl->handle,
-                             NV_CTRL_FORCE_GENERIC_CPU, enabled);
-
-    if (ret != NvCtrlSuccess) return;
-    
-    /*
-     * XXX the logic is awkward, but correct: when
-     * NV_CTRL_FORCE_GENERIC_CPU is enabled, use of enhanced CPU
-     * instructions is disabled, and vice versa.
-     */
-
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL use of enhanced CPU instructions %s.",
-                                 enabled ? "disabled" : "enabled");
+    post_xinerama_stereo_button_toggled(ctk_opengl, enabled);
 }
 
 static void aa_line_gamma_toggled(
@@ -669,11 +639,7 @@ static void aa_line_gamma_toggled(
                              NV_CTRL_OPENGL_AA_LINE_GAMMA, enabled);
 
     if (ret != NvCtrlSuccess) return;
-
-    ctk_config_statusbar_message(ctk_opengl->ctk_config,
-                                 "OpenGL gamma correction for antialiased "
-                                 "lines %s.",
-                                 enabled ? "enabled" : "disabled");
+    post_aa_line_gamma_toggled(ctk_opengl, enabled);
 }
 
 
@@ -697,30 +663,32 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
     case NV_CTRL_SYNC_TO_VBLANK:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->sync_to_vblank_button);
         func = G_CALLBACK(vblank_sync_button_toggled);
+        post_vblank_sync_button_toggled(ctk_opengl, event_struct->value);
         break;
     case NV_CTRL_FLIPPING_ALLOWED:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->allow_flipping_button);
         func = G_CALLBACK(allow_flipping_button_toggled);
+        post_allow_flipping_button_toggled(ctk_opengl, event_struct->value);
         break;
     case NV_CTRL_FORCE_STEREO:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->force_stereo_button);
         func = G_CALLBACK(force_stereo_button_toggled);
+        post_force_stereo_button_toggled(ctk_opengl, event_struct->value);
         break;
     case NV_CTRL_XINERAMA_STEREO:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->xinerama_stereo_button);
         func = G_CALLBACK(xinerama_stereo_button_toggled);
+        post_xinerama_stereo_button_toggled(ctk_opengl, event_struct->value);
         break;
     case NV_CTRL_OPENGL_AA_LINE_GAMMA:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->aa_line_gamma_button);
         func = G_CALLBACK(aa_line_gamma_toggled);
-        break;
-    case NV_CTRL_FORCE_GENERIC_CPU:
-        button = GTK_TOGGLE_BUTTON(ctk_opengl->force_generic_cpu_button);
-        func = G_CALLBACK(force_generic_cpu_toggled);
+        post_aa_line_gamma_toggled(ctk_opengl, event_struct->value);
         break;
     case NV_CTRL_SHOW_SLI_HUD:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->show_sli_hud_button);
         func = G_CALLBACK(show_sli_hud_button_toggled);
+        post_show_sli_hud_button_toggled(ctk_opengl, event_struct->value);
         break;
     default:
         return;
@@ -912,12 +880,6 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
                       "this option is set.");
     }
 
-    if (ctk_opengl->active_attributes & __FORCE_GENERIC_CPU) {
-        ctk_help_heading(b, &i, "Disable use of enhanced CPU "
-                         "instruction sets");
-        ctk_help_para(b, &i, __force_generic_cpu_help);
-    }
-    
     if (ctk_opengl->active_attributes & __SHOW_SLI_HUD) {
         ctk_help_heading(b, &i, "SLI Heads-Up-Display");
         ctk_help_para(b, &i, "This option draws information about the current "
