@@ -62,7 +62,7 @@ typedef enum {
     BANNER_ARTWORK_SOLARIS,
     BANNER_ARTWORK_THERMAL,
     BANNER_ARTWORK_TV,
-    BANNER_ARTWORK_VCSC,
+    BANNER_ARTWORK_VCS,
     BANNER_ARTWORK_X,
     BANNER_ARTWORK_XVIDEO,
     BANNER_ARTWORK_INVALID,
