@@ -18,6 +18,9 @@
 #include "NVCtrlLib.h"
 #include "nv_control.h"
 
+#define NVCTRL_EXT_NEED_CHECK          (XPointer)(~0)
+#define NVCTRL_EXT_NEED_NOTHING        (XPointer)(0)
+#define NVCTRL_EXT_NEED_TARGET_SWAP    (XPointer)(1)
 
 static XExtensionInfo _nvctrl_ext_info_data;
 static XExtensionInfo *nvctrl_ext_info = &_nvctrl_ext_info_data;
@@ -47,10 +50,46 @@ static /* const */ XExtensionHooks nvctrl_extension_hooks = {
 static XEXT_GENERATE_FIND_DISPLAY (find_display, nvctrl_ext_info,
                                    nvctrl_extension_name, 
                                    &nvctrl_extension_hooks,
-                                   NV_CONTROL_EVENTS, NULL)
+                                   NV_CONTROL_EVENTS, NVCTRL_EXT_NEED_CHECK)
 
 static XEXT_GENERATE_CLOSE_DISPLAY (close_display, nvctrl_ext_info)
 
+/*
+ * NV-CONTROL versions 1.8 and 1.9 pack the target_type and target_id
+ * fields in reversed order.  In order to talk to one of these servers,
+ * we need to swap these fields.
+ */
+static void XNVCTRLCheckTargetData(Display *dpy, XExtDisplayInfo *info,
+                                   int *target_type, int *target_id)
+{
+    /* Find out what the server's NV-CONTROL version is and
+     * setup for swapping if we need to.
+     */
+    if (info->data == NVCTRL_EXT_NEED_CHECK) {
+        int major, minor;
+
+        if (XNVCTRLQueryVersion(dpy, &major, &minor)) {
+            if (major == 1 &&
+                (minor == 8 || minor == 9)) {
+                info->data = NVCTRL_EXT_NEED_TARGET_SWAP;
+            } else {
+                info->data = NVCTRL_EXT_NEED_NOTHING;
+            }
+        } else {
+            info->data = NVCTRL_EXT_NEED_NOTHING;
+        }
+    }
+
+    /* We need to swap the target_type and target_id */
+    if (info->data == NVCTRL_EXT_NEED_TARGET_SWAP) {
+        int tmp;
+        tmp = *target_type;
+        *target_type = *target_id;
+        *target_id = tmp;
+    }
+}
+
+
 Bool XNVCTRLQueryExtension (
     Display *dpy,
     int *event_basep,
@@ -173,6 +212,7 @@ void XNVCTRLSetTargetAttribute (
     xnvCtrlSetAttributeReq *req;
 
     XNVCTRLSimpleCheckExtension (dpy, info);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
 
     LockDisplay (dpy);
     GetReq (nvCtrlSetAttribute, req);
@@ -255,6 +295,7 @@ Bool XNVCTRLQueryTargetAttribute (
         return False;
 
     XNVCTRLCheckExtension (dpy, info, False);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
 
     LockDisplay (dpy);
     GetReq (nvCtrlQueryAttribute, req);
@@ -308,6 +349,7 @@ Bool XNVCTRLQueryTargetStringAttribute (
         return False;
 
     XNVCTRLCheckExtension (dpy, info, False);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
 
     LockDisplay (dpy);
     GetReq (nvCtrlQueryStringAttribute, req);
@@ -417,6 +459,7 @@ Bool XNVCTRLQueryValidTargetAttributeValues (
         return False;
 
     XNVCTRLCheckExtension (dpy, info, False);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
 
     LockDisplay (dpy);
     GetReq (nvCtrlQueryValidAttributeValues, req);
@@ -1108,6 +1151,7 @@ Bool XNVCTRLQueryTargetBinaryData (
         return False;
 
     XNVCTRLCheckExtension (dpy, info, False);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
 
     LockDisplay (dpy);
     GetReq (nvCtrlQueryBinaryData, req);
@@ -1155,6 +1199,82 @@ Bool XNVCTRLQueryBinaryData (
                                         attribute, ptr, len);
 }
 
+Bool XNVCTRLStringOperation (
+    Display *dpy,
+    int target_type,
+    int target_id,
+    unsigned int display_mask,
+    unsigned int attribute,
+    char *pIn,
+    char **ppOut
+) {
+    XExtDisplayInfo *info = find_display(dpy);
+    xnvCtrlStringOperationReq *req;
+    xnvCtrlStringOperationReply rep;
+    Bool ret;
+    int inSize, outSize, length, slop;
+
+    if (!XextHasExtension(info))
+        return False;
+    
+    if (!ppOut)
+        return False;
+
+    *ppOut = NULL;
+    
+    XNVCTRLCheckExtension(dpy, info, False);
+    XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
+    
+    if (pIn) {
+        inSize = strlen(pIn) + 1;
+    } else {
+        inSize = 0;
+    }
+    
+    LockDisplay(dpy);
+    GetReq(nvCtrlStringOperation, req);
+    
+    req->reqType = info->codes->major_opcode;
+    req->nvReqType = X_nvCtrlStringOperation;
+    req->target_type = target_type;
+    req->target_id = target_id;
+    req->display_mask = display_mask;
+    req->attribute = attribute;
+
+    req->length += ((inSize + 3) & ~3) >> 2;
+    req->num_bytes = inSize;
+    
+    if (pIn) {
+        Data(dpy, pIn, inSize);
+    }
+    
+    if (!_XReply (dpy, (xReply *) &rep, 0, False)) {
+        UnlockDisplay(dpy);
+        SyncHandle();
+        return False;
+    }
+    
+    length = rep.length;
+    outSize = rep.num_bytes;
+    slop = outSize & 3;
+
+    if (outSize) *ppOut = (char *) Xmalloc(outSize);
+    
+    if (!*ppOut) {
+        _XEatData(dpy, length);
+    } else {
+        _XRead(dpy, (char *) *ppOut, outSize);
+        if (slop) _XEatData(dpy, 4-slop);
+    }
+    
+    ret = rep.ret;
+    
+    UnlockDisplay(dpy);
+    SyncHandle();
+    
+    return ret;
+}
+
 
 static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
 {
@@ -1163,6 +1283,8 @@ static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
     xnvctrlEvent *event;
     XNVCtrlEventTarget *reTarget;
     xnvctrlEventTarget *eventTarget;
+    XNVCtrlEventTargetAvailability *reTargetAvailability;
+    XNVCtrlStringEventTarget *reTargetString;
 
     XNVCTRLCheckExtension (dpy, info, False);
     
@@ -1171,7 +1293,7 @@ static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
         re = (XNVCtrlEvent *) host;
         event = (xnvctrlEvent *) wire;
         re->attribute_changed.type = event->u.u.type & 0x7F;
-        re->attribute_changed.serial = 
+        re->attribute_changed.serial =
             _XSetLastRequestRead(dpy, (xGenericReply*) event);
         re->attribute_changed.send_event = ((event->u.u.type & 0x80) != 0);
         re->attribute_changed.display = dpy;
@@ -1186,7 +1308,7 @@ static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
         reTarget = (XNVCtrlEventTarget *) host;
         eventTarget = (xnvctrlEventTarget *) wire;
         reTarget->attribute_changed.type = eventTarget->u.u.type & 0x7F;
-        reTarget->attribute_changed.serial = 
+        reTarget->attribute_changed.serial =
             _XSetLastRequestRead(dpy, (xGenericReply*) eventTarget);
         reTarget->attribute_changed.send_event =
             ((eventTarget->u.u.type & 0x80) != 0);
@@ -1204,6 +1326,52 @@ static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
         reTarget->attribute_changed.value =
             eventTarget->u.attribute_changed.value;
         break;
+    case TARGET_ATTRIBUTE_AVAILABILITY_CHANGED_EVENT:
+        reTargetAvailability = (XNVCtrlEventTargetAvailability *) host;
+        eventTarget = (xnvctrlEventTarget *) wire;
+        reTargetAvailability->attribute_changed.type =
+            eventTarget->u.u.type & 0x7F;
+        reTargetAvailability->attribute_changed.serial =
+            _XSetLastRequestRead(dpy, (xGenericReply*) eventTarget);
+        reTargetAvailability->attribute_changed.send_event =
+            ((eventTarget->u.u.type & 0x80) != 0);
+        reTargetAvailability->attribute_changed.display = dpy;
+        reTargetAvailability->attribute_changed.time =
+            eventTarget->u.availability_changed.time;
+        reTargetAvailability->attribute_changed.target_type =
+            eventTarget->u.availability_changed.target_type;
+        reTargetAvailability->attribute_changed.target_id =
+            eventTarget->u.availability_changed.target_id;
+        reTargetAvailability->attribute_changed.display_mask =
+            eventTarget->u.availability_changed.display_mask;
+        reTargetAvailability->attribute_changed.attribute =
+            eventTarget->u.availability_changed.attribute;
+        reTargetAvailability->attribute_changed.availability =
+            eventTarget->u.availability_changed.availability;
+        reTargetAvailability->attribute_changed.value =
+            eventTarget->u.availability_changed.value;
+        break;
+    case TARGET_STRING_ATTRIBUTE_CHANGED_EVENT:
+        reTargetString = (XNVCtrlStringEventTarget *) host;
+        eventTarget = (xnvctrlEventTarget *) wire;
+        reTargetString->attribute_changed.type = eventTarget->u.u.type & 0x7F;
+        reTargetString->attribute_changed.serial =
+            _XSetLastRequestRead(dpy, (xGenericReply*) eventTarget);
+        reTargetString->attribute_changed.send_event =
+            ((eventTarget->u.u.type & 0x80) != 0);
+        reTargetString->attribute_changed.display = dpy;
+        reTargetString->attribute_changed.time =
+            eventTarget->u.attribute_changed.time;
+        reTargetString->attribute_changed.target_type =
+            eventTarget->u.attribute_changed.target_type;
+        reTargetString->attribute_changed.target_id =
+            eventTarget->u.attribute_changed.target_id;
+        reTargetString->attribute_changed.display_mask =
+            eventTarget->u.attribute_changed.display_mask;
+        reTargetString->attribute_changed.attribute =
+            eventTarget->u.attribute_changed.attribute;
+        break;
+
     default:
         return False;
     }
